import { useEffect, useState } from 'react'
import './App.css';
import Checkerboard from './views/Checkerboard'
import RoomList from './views/RoomList'
import Login from './views/Login';
import RevMsgContext from './provider/RevMsgContext'
import GlobalSocketContext from './provider/GlobalSocketContext'
import RevRoomContext from './provider/RevRoomContext'
import { ROOM, HALL } from './Constant'
function App() {
  const [pageState, updatePageState] = useState(0)
  const [roomList, updateRoomList] = useState("")
  const [roomData, updateRoomData] = useState("")
  const [socket, setSocket] = useState(null)
  const [userName, setUsername] = useState("")

  const [scrollWidth, setScrollWidth] = useState(document.body.scrollWidth)
  const [scrollHeight, setScrollHeight] = useState(document.body.scrollHeight)

  useEffect(()=>{
    if ( scrollWidth > 1024 || scrollHeight > 1200 ) {
      document.querySelector("html").style.fontSize = "24px";
    } else {
      document.querySelector("html").style.fontSize = (scrollWidth / 375) * 10 + "px"
    }
  }, [scrollWidth, scrollHeight])

  useEffect(()=>{
    if ( userName === "" ) return;
    const socket = new WebSocket(`ws://39.105.160.25:11022/game/${userName}`) 
    socket.onopen = ()=>{
      socket.send(JSON.stringify( {instruct: HALL} ))
    }
    socket.onerror = ( error )=>{
      window.location.reload()
    }
    socket.onmessage = (msg)=>{
      const revMsg = JSON.parse(msg.data)
      switch ( revMsg.instruct ) {
        case HALL: updateRoomList(revMsg); return updatePageState(1)
        case ROOM: updateRoomData(revMsg); return updatePageState(2)
        default: return updatePageState(0)
      }
      
    }
    socket.onclose = (e)=>{
      
    }
    setSocket(socket)
    window.onresize = ()=>{
      setScrollWidth(document.body.scrollWidth)
      setScrollHeight(document.body.scrollHeight)
    }
  }, [userName])
  
  return (
    
      <GlobalSocketContext.Provider value={socket}>
        
        {
          (()=>{
            if ( pageState === 0 ) {
              return <Login setUsername={setUsername} />
            } else if ( pageState === 1 ) {
              return(
                <RevMsgContext.Provider value={roomList}>
                  <RoomList />
                </RevMsgContext.Provider>
              )
            } else if ( pageState === 2 ) {
              return (
                  <RevRoomContext.Provider value={roomData} >
                    <Checkerboard />
                  </RevRoomContext.Provider>
              )
            }
          })()
        }
      </GlobalSocketContext.Provider>
  );
}

export default App;
