import { useContext, useEffect, useState } from 'react'
import './Checkerboard.css'
import { POINT, WIN } from '../Constant';
import tableData from './CheckerboardData'
import Navigation from '../component/Navigation'
import RevRoomContext from '../provider/RevRoomContext';
import GlobalSocketContext from '../provider/GlobalSocketContext';
import Dialog from '../component/Dialog';
const bgline = [
  ["x", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n"],
  ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"]
];
const constantRev = {
  0: "x",
  1: "a",
  2: "b",
  3: "c",
  4: "d",
  5: "e",
  6: "f",
  7: "g",
  8: "h",
  9: "i",
  10: "j",
  11: "k",
  12: "l",
  13: "m",
  14: "n"
}
const Checkerboard = function () {
  const [dialogData, setDialog] = useState({
    title: "游戏提示",
    content: "xx获得游戏胜利!!",
    show: false
  })
  const { room, data } = useContext(RevRoomContext)
  const socket = useContext(GlobalSocketContext)
  const [table, updateTable] = useState(tableData)
  useEffect(() => {
    updateTable(data)
  }, [data])
  const goBangTouchEvent = function (x, y, value) {
    
    //判断对方是否赢了
    if ( room.winUsername !== "" ) {
      dialogMsg( `${room.winUsername} 赢得了此轮竞技` )
      return ;
    }

    if (room.status === 0) {
      dialogMsg( "此轮对弈还未开始呢!" )
      return
    }

    if (value === 0) {
      data[x][y] = room.status
      updateTable(data)
      socket.send(JSON.stringify({
        instruct: POINT,
        data: data
      }))
      if (room.status !== 0) {
        whoAreWin(data)
      }
    } else {
      console.log("此处已经落子了")
    }
  }
  const whoAreWin = (table) => {
    let blackWin = false
    let whiteWin = false
    //撇向
    for (let x = 5; x < 14; x++) {
      for (let y = 0; y <= 9; y++) {
        let white = 0
        let black = 0
        let step = 0
        while (step < 5) {
          //console.log(`坐标: ${x - step} ${y + step}`)
          if (table[constantRev[x - step]][y + step] !== 0) {
            if (table[constantRev[x - step]][y + step] === 1) black++
            if (table[constantRev[x - step]][y + step] === 2) white++
          }
          step++
        }
        if (black >= 5) blackWin = true
        if (white >= 5) whiteWin = true
      }
    }
    //捺向
    for (let x = 10; x > 0; x--) {
      for (let y = 0; y <= 9; y++) {
        let white = 0
        let black = 0
        let step = 0
        while (step < 5) {
          //console.log( `坐标: ${x + step} ${y + step}` )
          if (table[constantRev[x + step]][y + step] !== 0) {
            if (table[constantRev[x + step]][y + step] === 1) black++
            if (table[constantRev[x + step]][y + step] === 2) white++
          }
          step++
        }
        if (black >= 5) blackWin = true
        if (white >= 5) whiteWin = true
      }
    }
    //纵向
    for (let x = 1; x <= 14; x++) {
      for (let y = 0; y <= 9; y++) {
        let step = 0
        let white = 0
        let black = 0
        while (step < 5) {
          //console.log( `坐标: ${x} ${y + step}` )
          if (table[constantRev[x]][y + step] !== 0) {
            if (table[constantRev[x]][y + step] === 1) black++
            if (table[constantRev[x]][y + step] === 2) white++
          }
          step++
        }
        if (black >= 5) blackWin = true
        if (white >= 5) whiteWin = true
      }
    }
    //横向
    for (let y = 0; y <= 13; y++) {
      for (let x = 1; x <= 10; x++) {
        let step = 0
        let white = 0
        let black = 0
        while (step < 5) {
          //console.log( `坐标: ${y} ${x+step}` )
          if (table[constantRev[x + step]][y] !== 0) {
            if (table[constantRev[x + step]][y] === 1) black++
            if (table[constantRev[x + step]][y] === 2) white++
          }
          step++
        }
        if (black >= 5) blackWin = true
        if (white >= 5) whiteWin = true
      }
    }
    if (blackWin === true || whiteWin === true) {
      const who = blackWin === true ? room.backUsername : room.whiteUsername
      if ( who ) {
        socket.send(JSON.stringify({
          instruct: WIN,
          winUser: who
        }))
      }
      dialogMsg( `${who} 赢得了此轮竞技!` )
    }
  }
  const dialogMsg = (message) => {
    setDialog({
      ...dialogData,
      content: message,
      show: true,
      confirm: () => {
        setDialog({ ...dialogData, show: false })
      }
    })
  }
  return (
    <div className='body'>
      <Dialog {...dialogData} />
      <Navigation />
      <div className='center'>
        <div className='checkerboard'>
          <div className='board' >
            {Object.keys(table).map((x) => {
              return (
                <div key={x} className='board-grid'>
                  {table[x].map((value, y) => {
                    return (
                      <div
                        onClick={() => {
                          goBangTouchEvent(x, y, value)
                        }}
                        onTouchEnd={() => {
                          goBangTouchEvent(x, y, value)
                        }}
                        className={(() => {
                          switch (value) {
                            case 0: return 'board-item'
                            case 1: return 'black board-item'
                            case 2: return 'white board-item'
                            default: return 'board-item'
                          }
                        })()}
                        key={y}
                      >
                      </div>
                    )
                  })}
                </div>
              )
            })}
          </div>
          <div className='bj' >
            {(() => {
              const ret = bgline[0].map((x) => {
                return (
                  <div key={x} className='bj-grid'>
                    {
                      bgline[1].map((y) => {
                        return (
                          <div key={y} className={(() => {
                            let ret = 'bj-item'
                            if (x === 'n') {
                              ret += ' bj-bottom'
                            }
                            if (y === '14') {
                              ret += ' bj-right'
                            }
                            return ret
                          })()}>
                            {x === 'x' && y !== '0' ? <span className='xline'>{y}</span> : ''}
                            {y === '0' && x !== 'x' ? <span className='yline'>{x}</span> : ''}
                          </div>
                        )
                      })
                    }
                  </div>
                )
              })
              return ret
            })()}
          </div>
        </div>
      </div>
    </div>
  )
}
export default Checkerboard
